@@ -33,9 +33,7 @@ static void markMeshPoints(HXTMesh* mesh)
   }
 
   for(uint64_t i=0; i<mesh->points.num; i++) {
-    for(int j=0; j<2; j++) {
-      mesh->vertices.coord[4* mesh->lines.node[2*i+j] + 3] = 1.0;
-    }
+      mesh->vertices.coord[4* mesh->points.node[i] + 3] = 1.0;
   }
 }
 
@@ -62,16 +60,18 @@ HXTStatus hxtEmptyMesh(HXTMesh* mesh, HXTDelaunayOptions* delOptions)
     }
   }
 
+  delOptions->perfectDelaunay = 1;
   HXT_CHECK( hxtDelaunaySteadyVertices(mesh, delOptions, nodeInfo, numToInsert) );
+  delOptions->perfectDelaunay = 0;
 
-#ifdef DEBUG
+  //#ifdef DEBUG
   #pragma omp parallel for simd aligned(nodeInfo:SIMD_ALIGN)
   for (uint32_t i=0; i<numToInsert; i++) {
     if(nodeInfo[i].status!=HXT_STATUS_TRUE){
       HXT_WARNING("point %u of the empty mesh was not inserted\n", nodeInfo[i].node);
     }
   }
-#endif
+  //#endif
 
   HXT_CHECK( hxtAlignedFree(&nodeInfo) );
 
@@ -89,7 +89,7 @@ static int getBestCenter(double p[4][4], double nodalSize[4], double center[4],
   double num = 0;
   for(int i=0; i<4; i++) {
     double size = nodalSize[i];
-    if(size!=DBL_MAX && size>0.0) {
+    if(size>0.0) {
       avg += size;
       num+=1.0;
     }
@@ -103,10 +103,10 @@ static int getBestCenter(double p[4][4], double nodalSize[4], double center[4],
     avg /= num;
   }
 
-  double s0 = nodalSize[0]!=DBL_MAX && nodalSize[0]>0.0 ? nodalSize[0] : avg;
-  double s1 = nodalSize[1]!=DBL_MAX && nodalSize[1]>0.0 ? nodalSize[1] : avg;
-  double s2 = nodalSize[2]!=DBL_MAX && nodalSize[2]>0.0 ? nodalSize[2] : avg;
-  double s3 = nodalSize[3]!=DBL_MAX && nodalSize[3]>0.0 ? nodalSize[3] : avg;
+  double s0 = nodalSize[0]>0.0 ? nodalSize[0] : avg;
+  double s1 = nodalSize[1]>0.0 ? nodalSize[1] : avg;
+  double s2 = nodalSize[2]>0.0 ? nodalSize[2] : avg;
+  double s3 = nodalSize[3]>0.0 ? nodalSize[3] : avg;
 
   // (e/s)^2  (e is the norm of the edge, s is the mean nodalSize over that edge)
   double e0l2 = squareDist(p[0], p[1])/(0.25*(s0 + s1)*(s0 + s1));
@@ -127,9 +127,10 @@ static int getBestCenter(double p[4][4], double nodalSize[4], double center[4],
 
   // these formula accumulate a lot of errors...
   double xa = sqrt(e0l2);
-  double xb = (e1l2 + e0l2 - e3l2)/(2*xa);
+  double invtwoxa = 1./(2*xa);
+  double xb = (e1l2 + e0l2 - e3l2)*invtwoxa;
   double yb = sqrt(e1l2 - xb*xb);
-  double xc = (e2l2 + e0l2 - e4l2)/(2*xa);
+  double xc = (e2l2 + e0l2 - e4l2)*invtwoxa;
   double yc = (e1l2 + e2l2 - e5l2 - 2*xb*xc)/(2*yb);
   double zc = sqrt(e2l2 - xc*xc - yc*yc);
 
@@ -215,7 +216,7 @@ static int getBestCenter(double p[4][4], double nodalSize[4], double center[4],
     if(!isfinite(bary0) || !isfinite(bary1) || !isfinite(bary2) || !isfinite(bary3))
       bary0 = bary1 = bary2 = bary3 = 0.25;
 
-    // compute the cartesian coordinates from the barycentric ones
+    // compute the Cartesian coordinates from the barycentric ones
     double otherCenter[4];
     otherCenter[0] = bary0*p[0][0] + bary1*p[1][0] + bary2*p[2][0] + bary3*p[3][0];
     otherCenter[1] = bary0*p[0][1] + bary1*p[1][1] + bary2*p[2][1] + bary3*p[3][1];
@@ -286,7 +287,7 @@ static uint64_t* scanbsearch(uint64_t* array, uint64_t key, size_t num)
  *  3 the index of the first point it will create
  * To reserve the right number of points, we needs to know
  *  4 the total number of points
- * 
+ *
  *  1: startTet[t]
  *  2: startTet[t+1]  (startTet needs to be of size maxThreads+1)
  *  3: startPt[t]
@@ -297,11 +298,11 @@ static uint64_t* scanbsearch(uint64_t* array, uint64_t key, size_t num)
  *  - startTet[maxThreads] = mesh->tetrahedra.num
  *  - `startPt[t+1] - startPt[t]` gives how many point will be created by thread t
  */
-static HXTStatus balanceRefineWork(HXTMesh* mesh, uint32_t* startPt, size_t* startTet, int maxThreads)
+static HXTStatus balanceRefineWork(HXTMesh* mesh, uint32_t* startPt, uint64_t* startTet, int maxThreads)
 {
-  size_t* scan;
+  uint64_t* scan;
   uint32_t ptPerThreadGoal;
-  HXT_CHECK( hxtAlignedMalloc(&scan, sizeof(size_t) * mesh->tetrahedra.num) );
+  HXT_CHECK( hxtAlignedMalloc(&scan, sizeof(uint64_t) * mesh->tetrahedra.num) );
 
   #pragma omp parallel num_threads(maxThreads)
   {
@@ -343,7 +344,7 @@ static HXTStatus balanceRefineWork(HXTMesh* mesh, uint32_t* startPt, size_t* sta
 
     // we want to find i such that scan[i] = scanToFind.
     // we do a simple binary search in the prefix scan array to find `i`
-    size_t* pfnd = scanbsearch(scan, scanToFind, mesh->tetrahedra.num);
+    uint64_t* pfnd = scanbsearch(scan, scanToFind, mesh->tetrahedra.num);
     startTet[threadID] = pfnd  - scan;
     if(startTet[threadID] < mesh->tetrahedra.num)
       startPt[threadID] = *pfnd;
@@ -364,7 +365,7 @@ HXTStatus hxtRefineTetrahedra(HXTMesh* mesh,
   int maxThreads = omp_get_max_threads();
 
   uint64_t* startTet; // see balanceRefineWork
-  HXT_CHECK( hxtMalloc(&startTet, (maxThreads+1)*sizeof(size_t)) );
+  HXT_CHECK( hxtMalloc(&startTet, (maxThreads+1)*sizeof(uint64_t)) );
 
   uint32_t* startPt; // see balanceRefineWork
   HXT_CHECK( hxtMalloc(&startPt, (maxThreads+1)*sizeof(uint32_t)) );
@@ -414,8 +415,20 @@ HXTStatus hxtRefineTetrahedra(HXTMesh* mesh,
     }
 
     // second step (meshSizeCB): compute the effective mesh size at all these newly create points
-    if(delOptions->nodalSizes->callback!=NULL)
-      HXT_CHECK( delOptions->nodalSizes->callback(newVertices, totNewPts, delOptions->nodalSizes->userData) );
+    if(delOptions->nodalSizes->callback!=NULL) {
+
+      // give the colors too
+      uint32_t* newPtColors;
+      HXT_CHECK( hxtMalloc(&newPtColors, sizeof(uint32_t) * totNewPts) );
+
+      for(uint64_t i=0; i<totNewPts; i++) {
+        newPtColors[i] = mesh->tetrahedra.color[ptToTet[i]];
+      }
+
+      HXT_CHECK( delOptions->nodalSizes->callback(newVertices, newPtColors, totNewPts, delOptions->nodalSizes->userData) );
+
+      HXT_CHECK( hxtFree(&newPtColors) );
+    }
 
 
     HXTStatus status;
@@ -427,7 +440,7 @@ HXTStatus hxtRefineTetrahedra(HXTMesh* mesh,
       // we don't really need startPt and startTet anymore because points have been created
       // we recycle startPt to count how many point will be kept for each thread after
       // the filtering
-      startPt[threadID] = 0; 
+      startPt[threadID] = 0;
 
       #pragma omp for schedule(static)
       for(size_t i=0; i<totNewPts; i++) {
@@ -441,7 +454,7 @@ HXTStatus hxtRefineTetrahedra(HXTMesh* mesh,
 
         double* vtaCoord = &newVertices[4*i];
         double vtaSize = newVertices[4*i + 3];
-        
+
         if(isTooClose(s[0], vtaSize, squareDist(p[0], vtaCoord), delOptions->nodalSizes) ||
            isTooClose(s[1], vtaSize, squareDist(p[1], vtaCoord), delOptions->nodalSizes) ||
            isTooClose(s[2], vtaSize, squareDist(p[2], vtaCoord), delOptions->nodalSizes) ||
