@@ -641,7 +641,7 @@ static void Mesh2D(GModel *m)
       }
     }
     bool debug = (Msg::GetVerbosity() == 99);
-    
+
     transferSeamGEdgesVerticesToGFace(m);
     quadMeshingOfSimpleFacesWithPatterns(m,.2);
     if(debug) m->writeMSH("opti1.msh");
@@ -651,7 +651,7 @@ static void Mesh2D(GModel *m)
     //for(GFace *gf : m->getFaces()) if(debug) gf->model()->writeMSH("opti3.msh");
     OptimizeMesh(m, "UntangleTris");
     if(debug) m->writeMSH("opti4.msh");
-    
+
     for(GFace *gf : m->getFaces()) {
       if(gf->meshStatistics.status == GFace::PENDING) {
         gf->meshStatistics.status = GFace::DONE;
@@ -810,6 +810,8 @@ static void Mesh3D(GModel *m)
     }
   }
 
+  bool hexDominant = false;
+
   for(std::size_t i = 0; i < connected.size(); i++) {
     if(CTX::instance()->abortOnError && Msg::GetErrorCount()) {
       Msg::Warning("Aborted 3D meshing");
@@ -838,6 +840,7 @@ static void Mesh3D(GModel *m)
 
       if(treat_region_ok && (CTX::instance()->mesh.recombine3DAll ||
                              gr->meshAttributes.recombine3D)) {
+        hexDominant = true;
 	meshCombine3D(gr);
         RelocateVertices(gr, CTX::instance()->mesh.nbSmoothing);
       }
@@ -845,7 +848,9 @@ static void Mesh3D(GModel *m)
 #endif
   }
 
-  MakeHybridHexTetMeshConformalThroughTriHedron(m);
+  if(hexDominant)
+    MakeHybridHexTetMeshConformalThroughTriHedron(m);
+
   // ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
@@ -892,7 +897,7 @@ void OptimizeMesh(GModel *m, const std::string &how, bool force, int niter)
     for(auto it = m->firstRegion(); it != m->lastRegion(); it++) {
       optimizeMeshGRegion opt;
       opt(*it, force);
-      // RelocateVerticesOfPyramids(*it, 10, 1.e-6);      
+      // RelocateVerticesOfPyramids(*it, 10, 1.e-6);
     }
     m->setAllVolumesPositive();
   }
@@ -1498,6 +1503,12 @@ void GenerateMesh(GModel *m, int ask)
     // Mesh2D(m);
   }
 
+  if(m->getMeshStatus() >= 2 && CTX::instance()->mesh.optimizePyramids < 0.) {
+    // make sure surface mesh is correctly oriented before meshing in 3D if we
+    // push points along the normals to create pyramids
+    std::for_each(m->firstFace(), m->lastFace(), orientMeshGFace());
+  }
+
   // 3D mesh
   if(ask == 3) { Mesh3D(m); }
 
